#!/bin/bash

# echo commands
set -x
# exit on error
set -e

# save folder of script for location of the patches
patch_folder=$(readlink -f $(dirname $0))

# install necessary system packages
sudo apt-get install --assume-yes git cmake g++ libboost-all-dev bc flex bison libreadline-dev ocaml ocamlbuild ocaml-native-compilers locales ghc haskell-stack openjdk-9-jdk gradle

# generate local for number formats
sudo locale-gen en_US.UTF-8

# collect binaries in this folder
cd ~
mkdir -p ~/bin
export PATH=$PATH:$HOME/bin
echo "export PATH=\$PATH:\$HOME/bin" >>~/.bashrc

# install combine-aiger (use sparse checkout)
cd ~
mkdir syntcomp
cd syntcomp
git init
git remote add -f origin https://bitbucket.org/swenjacobs/syntcomp.git
git config core.sparsecheckout true
echo "tools/aiger-ltl-model-checker/" >> .git/info/sparse-checkout
git pull origin master
cd tools/aiger-ltl-model-checker
make
cp combine-aiger ~/bin
cd ~
rm -rf syntcomp

# install syfco
cd ~
git clone https://github.com/reactive-systems/syfco.git
cd syfco
make
cp syfco ~/bin
cd ~
rm -rf syfco

# install abc
git clone https://github.com/berkeley-abc/abc.git
cd abc
make
cp abc ~/bin
rm -rf abc

# install speculoos dependencies
cd ~
git clone https://github.com/romainbrenguier/ocaml-cudd.git
git clone https://github.com/romainbrenguier/ocaml-aiger.git
patch -p0 <installation/ocaml_cudd.patch
cd ocaml-cudd
make
cp cudd_stub.mli cudd_stub.ml
ocamlbuild cudd_stub.cmo
cd ~
cd ocaml-aiger
make
cd ~

# install Speculoos
cd ~
git clone https://github.com/romainbrenguier/Speculoos
patch -p0 <$patch_folder/speculoos.patch
cd Speculoos/src
(make || true)
cd _build
/usr/bin/ocamlc.opt ../../../ocaml-cudd/_build/cudd_stub.cmo ../../../ocaml-aiger/_build/aiger.cma ../../../ocaml-aiger/_build/aigerImperative.cma -custom ../../../ocaml-cudd/_build/cudd.o ../../../ocaml-cudd/_build/cudd.cmo ../../../ocaml-cudd/cudd-2.5.0/cudd/libcudd.a ../../../ocaml-cudd/cudd-2.5.0/util/libutil.a ../../../ocaml-cudd/cudd-2.5.0/epd/libepd.a ../../../ocaml-cudd/cudd-2.5.0/mtr/libmtr.a ../../../ocaml-cudd/cudd-2.5.0/st/libst.a aigerBdd.cmo bddVariable.cmo common.cmo boolean.cmo integer.cmo type.cmo expression.cmo exprToBdd.cmo synthesis.cmo aigerImpBdd.cmo synthesisImp.cmo speculoos.cmo parser.cmo speculoosCompiler.cmo -warn-error -a-31 -o speculoosCompiler.byte
cp speculoosCompiler.byte ~/bin/speculoos
cd ~
rm -rf Speculoos
rm -rf ocaml-cudd
rm -rf ocaml-aiger

# install NuSMV
firefox http://nusmv.fbk.eu/NuSMV/download/getting_src-v2.html &
echo "Please download NuSMV from the displayed page"
echo "Afterwards, there should be a file 'NuSMV-2.6.0.tar.gz' in the 'Downloads' folder"
read -p "Then, press [Enter] to continue with the installation..."
cd ~
tar -xzf Downloads/NuSMV-2.6.0.tar.gz
patch -p0 <$patch_folder/nusmv_minisat.patch
patch -p0 <$patch_folder/nusmv_cudd.patch
cd NuSMV-2.6.0/NuSMV
mkdir build
cd build
cmake ..
make
cp bin/ltl2smv ~/bin
cd ~
rm -r NuSMV-2.6.0

# install nuXmv
firefox https://es-static.fbk.eu/tools/nuxmv/index.php?n=Download.Download &
echo "Please download nuXmv from the displayed page"
echo "Afterwards, there should be a file 'nuXmv-1.1.1-linux64.tar.gz' in the 'Downloads' folder"
read -p "Then, press [Enter] to continue with the installation..."
cd ~
tar -xzf Downloads/nuXmv-1.1.1-linux64.tar.gz nuXmv-1.1.1-Linux/bin/nuXmv
cp nuXmv-1.1.1-Linux/bin/nuXmv ~/bin
rm -r nuXmv-1.1.1-Linux

# install aiger tools
wget http://fmv.jku.at/aiger/aiger-1.9.4.tar.gz
tar -xzf aiger-1.9.4.tar.gz
cd aiger-1.9.4
./configure
make
cp aigtoaig smvtoaig ../bin
cd ~
rm -r aiger-1.9.4 aiger-1.9.4.tar.gz

# install MeMin
wget http://embedded.cs.uni-saarland.de/tools/MeMin/MeMin -O ~/bin/MeMin
chmod +x ~/bin/MeMin

# install owl
cd ~
cd owl
./gradlew jar
cp build/libs/owl-1.2.0-SNAPSHOT.jar lib
for jar in jhoafparser-1.1.1.jar jbdd-0.2.0.jar guava-23.4-jre.jar naturals-util-0.8.0.jar fastutil-8.1.1.jar commons-cli-1.4.jar antlr4-runtime-4.7.jar; do
    find ~/.gradle -name "$jar" -exec cp {} lib \;
done
cd ..

# install strix
cd ~
cd strix
make
cp bin/strix ~/bin
cp bin/kisstospec ~/bin
cp run_strix.sh ~/bin
cd ~

set +x

echo "Everything went fine!"
echo "Use the 'strix' command to invoke the base synthesis engine"
echo "Use the 'run_strix.sh' command to invoke the complete toolchain"
echo "Both commands can be used with '--help' to display usage options"
