/*
 * Copyright (C) 2016  (See AUTHORS)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package owl.ltl;

import java.util.BitSet;
import java.util.Objects;
import owl.ltl.visitors.BinaryVisitor;
import owl.ltl.visitors.IntVisitor;
import owl.ltl.visitors.Visitor;

/**
 * Globally.
 */
public class GOperator extends UnaryModalOperator {

  public GOperator(Formula f) {
    super(f);
  }

  public static Formula of(Formula operand) {
    if (operand instanceof BooleanConstant) {
      return operand;
    }

    if (operand instanceof GOperator) {
      return operand;
    }

    if (operand instanceof FOperator && ((FOperator) operand).operand instanceof GOperator) {
      return operand;
    }

    if (operand instanceof ROperator) {
      return of(((ROperator) operand).right);
    }

    if (operand instanceof Conjunction) {
      return Conjunction.of(((Conjunction) operand).children.stream().map(GOperator::of));
    }

    return new GOperator(operand);
  }

  @Override
  public int accept(IntVisitor v) {
    return v.visit(this);
  }

  @Override
  public <R> R accept(Visitor<R> v) {
    return v.visit(this);
  }

  @Override
  public <A, B> A accept(BinaryVisitor<B, A> v, B parameter) {
    return v.visit(this, parameter);
  }

  @Override
  public String getOperator() {
    return "G";
  }

  @Override
  protected int hashCodeOnce() {
    return Objects.hash(GOperator.class, operand);
  }

  @Override
  public boolean isPureEventual() {
    return operand.isPureEventual();
  }

  @Override
  public boolean isPureUniversal() {
    return true;
  }

  @Override
  public boolean isSuspendable() {
    return operand.isPureEventual() || operand.isSuspendable();
  }

  @Override
  public UnaryModalOperator not() {
    return new FOperator(operand.not());
  }

  @Override
  public Formula unfold() {
    return new Conjunction(operand.unfold(), this);
  }

  @Override
  public Formula unfoldTemporalStep(BitSet valuation) {
    return Conjunction.of(operand.unfoldTemporalStep(valuation), this);
  }
}
