#!/bin/bash

if [ -z $1 ] || [ -z $2 ] || [ -z $3 ]; then
    echo "Usage: $0 MEMLMIT TIMELIMIT BENCHMARK"
    echo "MEMLIMIT  is the memory limit in GB."
    echo "TIMELIMIT is the time limit in seconds for each instance"
    echo "            and each realizability/synthesis and verification step."
    echo "BENCHMARK is the folder containing the benchmark instances."
    exit 1
fi

if [[ ! $1 =~ ^[1-9][0-9]*$ ]]; then
    echo "Error MEMLIMIT has to be a positive integer: $1"
    exit 1
fi

if [[ ! $2 =~ ^[1-9][0-9]*$ ]]; then
    echo "Error TIMELIMIT has to be a positive integer: $2"
    exit 1
fi

if [ ! -d $3 ]; then
    echo "Error BENCHMARK does not exist or is not a folder: $3"
    exit 1
fi

result_file=results.csv
out_dir=results_out

synthesis=true
verification=true
memorylimit=$1
timelimit=$2
benchmark=$3

# use this option for the syntcomp 2017 benchmarks
#benchmark='benchmarks/SYNTCOMP2017'
# uncomment this option to synthesize the arbiter for the amba case study
#benchmark='benchmarks/SYNTCOMP2016'

OPTIONS="--verbose --timelimit $timelimit --memorylimit $memorylimit --output $out_dir --benchmark $result_file"
if [ "$synthesis" == true ]; then
    OPTIONS="$OPTIONS"
else
    OPTIONS="$OPTIONS --realizability"
fi
if [ "$verification" == true ]; then
    OPTIONS="$OPTIONS --verify"
fi

echo -n "file,result_realizability,time_realizability" >$result_file
if [ "$synthesis" == true ]; then
    echo -n ",result_synthesis,time_synthesis,size_mealy,size_min_mealy,size_aiger,size_min_aiger,size_reduced_aiger,size_min_reduced_aiger" >>$result_file
fi
if [ "$verification" == true ]; then
    echo -n ",result_verification,time_verification" >>$result_file
fi
echo >>$result_file

mkdir -p $out_dir

for FILE in $(find $benchmark -mindepth 1 -type f -name "*.tlsf" | sort); do
    echo "Benchmarking $FILE"
    ./run_strix.sh $OPTIONS $FILE
done
