#pragma once

#include <iostream>
#include <limits>

#include "owl.h"
#include "ltl/TLSF.h"
#include "Quine.h"

namespace controller {

typedef unsigned int state_id_t;
constexpr state_id_t TOP_STATE = std::numeric_limits<state_id_t>::max();

class MealyMachine {
private:
    struct Successor {
        state_id_t successor;
        ternary_int<ltl::letter_t> output;

        Successor() : successor(0), output(0) {}
        Successor(const state_id_t successor, const ternary_int<ltl::letter_t> output) :
            successor(successor), output(output) {}
    };
    const ltl::TLSF& tlsf;
    const ltl::letter_t n_inputs;

    state_id_t n_states;
    state_id_t n_transitions;
    std::vector<Successor> successors;
public:

    MealyMachine(const ltl::TLSF& tlsf) :
        tlsf(tlsf),
        n_inputs(1 << tlsf.n_inputs),
        n_states(0),
        n_transitions(0)
    { }
    ~MealyMachine() { }

    state_id_t addState();
    state_id_t getNumberOfStates() const;
    void setSuccessor(const state_id_t s, const ltl::letter_t input, const state_id_t successor, const std::vector<ltl::letter_t>& outputs);

    void reduce();

    void print_spec(std::ostream& out) const;
    void print_kiss(std::ostream& out) const;
};

}
