#pragma once

#include "pg/PGArena.h"
#include "pg/PGSolver.h"
#include "ocl/OCLDriver.h"
#include "ocl/OCLPGSolverDefinitions.h"

namespace ocl {

class OCLPGSolver : public pg::PGSolver {
private:
    OCLDriver& driver;

    OCLKernel* kernel_strategy_iteration_init;
    OCLKernel* kernel_update_nodes;
    OCLKernel* kernel_bellman_ford_init;
    OCLKernel* kernel_bellman_ford_sys_iteration;
    OCLKernel* kernel_bellman_ford_env_iteration;
    OCLKernel* kernel_strategy_improvement;

    compute::vector<edge_id_t>* sys_succs_begin_buffer;
    compute::vector<node_id_t>* sys_succs_buffer;
    compute::vector<color_t>* colors_buffer;
    compute::vector<distance_t>* sys_distance_buffer;
    compute::vector<distance_t>* env_distance_buffer;

    compute::buffer* realizable_buffer;

    void strategy_iteration();
    void bellman_ford();

    inline void print_values_debug();

    void init();
    void clean();

protected:
    void reset();
    void solve_game();

public:
    OCLPGSolver(OCLDriver& driver, pg::PGArena& arena, const bool verbose = false);
    ~OCLPGSolver();
};

}
