#pragma once

#include <atomic>
#include <mutex>
#include <condition_variable>

#include "ltl/TLSF.h"
#include "PGArenaDefinitions.h"
#include "ParityAutomatonTree.h"

namespace pg {

class PGArena {
private:
    const ltl::TLSF& tlsf;
    std::unique_ptr<ParityAutomatonTree> ptree;

    bool has_lock;

    void buildProductAutomaton(const size_t k, const bool parallel = false, const bool print_product_states = false, const bool print_product_successors = false);

public:
    PGArena(const ltl::TLSF& tlsf, const owl::LabelledTree<owl::Tag, owl::Automaton>& tree);
    ~PGArena();

    void constructArena(const size_t k, const bool parallel = false, const bool print_product_states = false, const bool priiint_product_successors = false);

    // mutex for reading from or resizing the arena vectors
    std::mutex arena_mutex;

    // mutex for accessing size of the arena
    std::mutex size_mutex;

    // condition variable signalling change in size of the arena
    std::condition_variable change;

    // variable signalling that the arena is completely constructed
    std::atomic<bool> complete;

    // variable signalling that the arena is already solved
    std::atomic<bool> solved;

    const Parity parity_type;
    const color_t n_colors;

    const node_id_t initial_node;

    const ltl::letter_t n_env_actions;

    std::atomic<node_id_t> n_env_nodes;
    std::atomic<edge_id_t> n_sys_edges;

    std::vector<edge_id_t> sys_succs_begin;
    std::vector<node_id_t> sys_succs;

    std::vector<ltl::letter_t> sys_output;
    std::vector<color_t> colors;

    void print(std::ostream& out) const;
    void print_basic_info() const;
};

}
