#pragma once

#include "pg/PGArena.h"
#include "pg/PGSolver.h"

namespace pg {

typedef int distance_t;
typedef unsigned int dist_id_t;

constexpr distance_t DISTANCE_SOLVED = std::numeric_limits<distance_t>::max();
constexpr distance_t DISTANCE_INFINITY = std::numeric_limits<distance_t>::max() - 1;

class PGSISolver : public PGSolver {
private:
    std::vector<distance_t> sys_distances;
    std::vector<distance_t> env_distances;

    inline distance_t color_distance_delta(const color_t& color);

    void strategy_iteration();

    void update_nodes();

    void bellman_ford();
    void bellman_ford_init();
    void bellman_ford_sys_iteration();
    bool bellman_ford_env_iteration();

    bool strategy_improvement();

    inline void print_values_debug();

protected:
    void reset();
    void solve_game();

public:
    PGSISolver(PGArena& arena, const bool parallel_construction, const int num_threads, const bool verbose = false);
    ~PGSISolver();

};

}
