#pragma once

#include "controller/MealyMachine.h"
#include "pg/PGArena.h"

namespace pg {

class PGSolver {
private:
    void init_solver();

protected:
    pg::PGArena& arena;
    const bool parallel_construction;
    const int num_threads;
    const bool verbose;

    std::vector<uint8_t> successors;
    bool realizable;

    node_id_t n_env_nodes;
    node_id_t n_sys_nodes;
    node_id_t n_sys_edges;

    color_t n_colors;
    std::vector<color_t> color_map;

    void compact_colors();

    virtual void reset() = 0;
    virtual void solve_game() = 0;

public:
    PGSolver(pg::PGArena& arena, const bool parallel_construction, const int num_threads, const bool verbose = false);
    virtual ~PGSolver();

    virtual void solve();

    bool is_realizable() const;
    void construct_automaton(controller::MealyMachine& m) const;

    void print_debug(const std::string& str) const;
};

}
