#pragma once

#include <boost/program_options.hpp>
#include <boost/algorithm/string.hpp>

#include "owl.h"

namespace po = boost::program_options;

namespace strix {

enum class InputFileFormat { LTL, TLSF };

enum class OutputFileFormat { SPEC, KISS };

struct Options {
    std::string input_file;
    InputFileFormat input_file_format;
    OutputFileFormat output_file_format;
    std::string output_file;
    size_t work_group_size;
    size_t increment;
    bool solve_game;
    bool parallel;
    int threads;
    owl::SafetySplitting splitting;
    bool print_arena;
    bool print_device_info;
    bool print_arena_info;
    bool print_formula;
    bool print_tree;
    bool print_product_states;
    bool print_product_successors;
    bool timing;
    bool help;
    bool compact_colors;
    bool monolithic;
    bool realizability;
    bool validate_jni;
    bool verbose_solver;
};

class OptionParser {
private:
    po::options_description help_options_desc_;
    po::options_description hidden_options_desc_;
    po::positional_options_description positional_options_desc_;
    po::options_description options_desc_;

public:
    OptionParser();
    ~OptionParser();

    void print_options() const;
    Options parse_options(const int argc, const char* argv[]) const;
};

}
