#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

int main(int argc, char* argv[]) {
    if (argc <= 2) {
        printf("Usage: %s INPUT OUTPUT\n", argv[0]);
        return 1;
    }
    FILE* file = fopen(argv[1], "r");
    if (!file) {
        printf("Could not open input file\n");
        return 1;
    }
    FILE* outfile = fopen(argv[2], "w");
    if (!outfile) {
        printf("Could not open output file\n");
        return 1;
    }

    char inputs[1024][1024];
    char outputs[1024][1024];
    unsigned int n_inputs = 0;
    unsigned int n_outputs = 0;
    unsigned int i, o;

    char line[1024];
    char* tokens;

    // read inputs
    fgets(line, sizeof(line), file);
    tokens = strtok(line, " \n");
    tokens = strtok(NULL, " \n");
    while (tokens != NULL) {
        strcpy(inputs[n_inputs], tokens);
        n_inputs++;
        tokens = strtok(NULL, " \n");
    }

    // read outputs
    fgets(line, sizeof(line), file);
    tokens = strtok(line, " \n");
    tokens = strtok(NULL, " \n");
    while (tokens != NULL) {
        strcpy(outputs[n_outputs], tokens);
        n_outputs++;
        tokens = strtok(NULL, " \n");
    }

    // skip number of inputs/outputs/products
    fgets(line, sizeof(line), file);
    fgets(line, sizeof(line), file);
    fgets(line, sizeof(line), file);

    // get number of states
    fgets(line, sizeof(line), file);
    tokens = strtok(line, " \n");
    tokens = strtok(NULL, " \n");
    unsigned short n_states = strtol(tokens, NULL, 10);
    unsigned short log_n_states = ceil(log2(n_states));
    if (n_states > 32768) {
        printf("Too many states: %u\n", n_states);
        return 1;
    }

    unsigned short state_map[32768];

    // get reset state
    fgets(line, sizeof(line), file);
    tokens = strtok(line, " \n");
    tokens = strtok(NULL, " \n");
    unsigned short reset_state = strtol(tokens + 1, NULL, 10);
    state_map[reset_state] = 0;
    unsigned short state = 1;
    for (i = 0; i < n_states; ++i) {
        if (i != reset_state) {
            state_map[i] = state;
            state++;
        }
    }

    // print variables
    if (n_states > 1) {
        fprintf(outfile, "var state : int %u;\n", log_n_states);
    }
    for (i = 0; i < n_inputs; ++i) {
        fprintf(outfile, "var %s : bool;\n", inputs[i]);
    }
    for (o = 0; o < n_outputs; ++o) {
        fprintf(outfile, "var %s : bool;\n", outputs[o]);
    }
    fprintf(outfile, "\n");

    // print transitions
    while (fgets(line, sizeof(line), file)) {
        char* ptr = strtok(line, " \n");

        if (n_inputs > 0 || n_states > 1) {
            fprintf(outfile, "when ( ");
        }
        if (n_inputs > 0) {
            for (i = 0; i < n_inputs; ++i) {
                if (*ptr == '0' || *ptr == '1') {
                    if (*ptr == '0') {
                        fprintf(outfile, "!");
                    }
                    else if (*ptr == '1') {
                        fprintf(outfile, " ");
                    }
                    fprintf(outfile, "%s", inputs[i]);
                    if (n_states > 1 || i + 1 < n_inputs) {
                        fprintf(outfile, " & ");
                    }
                }
                ptr++;
            }
            ptr = strtok(NULL, " \n");
        }
        unsigned short s_old = strtol(ptr + 1, &ptr, 10);
        ptr = strtok(NULL, " \n");
        unsigned short s_new = strtol(ptr + 1, &ptr, 10);
        ptr = strtok(NULL, " \n");
        if (n_states > 1) {
            fprintf(outfile, "state = %u", state_map[s_old]);
        }
        if (n_inputs > 0 || n_states > 1) {
            fprintf(outfile, " ) { ");
        }
        if (n_states > 1) {
            fprintf(outfile, "state <- %u; ", state_map[s_new]);
        }
        for (o = 0; o < n_outputs; ++o) {
            if (*ptr == '0' || *ptr == '1') {
                fprintf(outfile, "%s <- %c; ", outputs[o], *ptr);
            }
            ptr++;
        }
        if (n_inputs > 0 || n_states > 1) {
            fprintf(outfile, "}");
        }
        fprintf(outfile, "\n");
    }

    if (!feof(file)) {
        printf("Error reading file\n");
        return 1;
    }
    fclose(file);
    fclose(outfile);

    return 0;
}
