#!/bin/bash

# exit on error
set -e
# echo commands
set -x
# break when pipe fails
set -o pipefail

# tool paths
STRIX=$1
SYFCO=syfco
SMVTOAIG=smvtoaig
AIGTOAIG=aigtoaig
SPECULOOS=speculoos
COMBINEAIGER=combine-aiger
ABC=abc
NUXMV=nuXmv
LTL2SMV=ltl2smv
MEMIN=MeMin
KISSTOSPEC=kisstospec

# input file
TLSF_FILE=$2
BASE_FILE=$(basename ${2%.tlsf})

# temporary files
CONTROLLER_KISS_FILE=/tmp/$BASE_FILE.kiss
CONTROLLER_MIN_KISS_FILE=/tmp/$BASE_FILE.min.kiss
CONTROLLER_SPEC_FILE=/tmp/$BASE_FILE.spec
CONTROLLER_AAG_FILE=/tmp/$BASE_FILE.aag
CONTROLLER_AIG_FILE=/tmp/$BASE_FILE.aig
CONTROLLER_RED_AIG_FILE=/tmp/$BASE_FILE.red.aig
MONITOR_AIGER_FILE=/tmp/$BASE_FILE.monitor.aag
COMBINED_FILE=/tmp/$BASE_FILE.combined.aag
SMV_FILE=/tmp/$BASE_FILE.smv
RESULT_FILE=/tmp/$BASE_FILE.result
OUT_FILE=/tmp/$BASE_FILE.out

# type of test
TEST=$3

# run the tool
$STRIX -i tlsf $TLSF_FILE -p --threads 8 -o $CONTROLLER_KISS_FILE -f kiss >$OUT_FILE

if [[ "$TEST" == "REALIZABLE" || "$TEST" == "SYNTHESIS" ]]; then
    # check if tool answers realizable
    grep -q "^REALIZABLE$" $OUT_FILE

    if [[ "$TEST" == "SYNTHESIS" ]]; then
        # build aiger circuit
        # $MEMIN $CONTROLLER_KISS_FILE
        # head -n 2 $CONTROLLER_KISS_FILE >$CONTROLLER_MIN_KISS_FILE
        # cat result.kiss >>$CONTROLLER_MIN_KISS_FILE
        # rm result.kiss
        cp $CONTROLLER_KISS_FILE $CONTROLLER_MIN_KISS_FILE
        $KISSTOSPEC $CONTROLLER_MIN_KISS_FILE $CONTROLLER_SPEC_FILE
        $SPECULOOS $CONTROLLER_SPEC_FILE -o $CONTROLLER_AAG_FILE
        $AIGTOAIG $CONTROLLER_AAG_FILE $CONTROLLER_AIG_FILE
        # /bin/echo -e "read_aiger $CONTROLLER_AIG_FILE\nrefactor\nrewrite\nwrite_aiger -s $CONTROLLER_RED_AIG_FILE" | $ABC
        cp $CONTROLLER_AIG_FILE $CONTROLLER_RED_AIG_FILE

        # build a monitor for the formula
        $SYFCO -f smv -m fully -os mealy $TLSF_FILE >$SMV_FILE
        $SMVTOAIG -L $LTL2SMV $SMV_FILE >$MONITOR_AIGER_FILE
        $COMBINEAIGER $MONITOR_AIGER_FILE $CONTROLLER_RED_AIG_FILE >$COMBINED_FILE

        # model check solution from tool
        /bin/echo -e "read_aiger_model -i ${COMBINED_FILE}\ngo\ncheck_ltlspec\nquit" | $NUXMV -int >$RESULT_FILE

        # check result
        if grep -q 'specification .* is true' $RESULT_FILE; then
            exit 0
        else
            exit 1
        fi
    fi
elif [[ "$TEST" == "UNREALIZABLE" ]]; then
    # check if tool answers unrealizable
    grep -q "^UNREALIZABLE$" $OUT_FILE
else
    # unknown test
    echo "Unknown test: $TEST"
    exit 2
fi
